/* 
 * Copyright (c) 2010, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/*
 *  ======== main_host_HLOS.c ========
 */

/* xdctools header files */
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/knl/Thread.h>

/* package header files */
#include <ti/sdo/rcm/examples/appcfg/App.h>
#include <ti/sdo/rcm/examples/appcfg/SystemCfg.h>

/* local header files */
#include "Global.h"

/*
 *  ======== main ========
 */
#define FXNN "main"
Int main(Int argc, Char* argv[])
{
    Int status, startStat, resStat;
    SystemCfg_Params systemCfgP;
    SystemCfg_Handle systemCfgH;


    /* initialize local vars */
    status = 0;
    systemCfgH = NULL;

    /* must initialize a module before using it */
    SystemCfg_init();

    /* start all statically created threads now */
    Thread_start(NULL);

    /* create a SystemCfg instance for the remote processor */
    SystemCfg_Params_init(&systemCfgP);
    systemCfgP.remoteProcName = Global_ServerProcName;

    status = SystemCfg_create(&systemCfgP, &systemCfgH);

    if (status < 0) {
        goto leave;
    }

    /* bring up the remote processor */
    startStat = SystemCfg_start(systemCfgH, Global_ServerExecutable);

    if (startStat < 0) {
        status = -1;
        goto leave;
    }

    /* create local resources, some might be shared */
    resStat = SystemCfg_initializeResources(systemCfgH, NULL, NULL);

    if (resStat < 0) {
        status = -1;
        goto leave;
    }

    /* invoke the application entry point */
    status = App_exec(NULL);

    if (status < 0) {
        goto leave;
    }


leave:
    /* start the shutdown process */
    if (resStat >= 0) {
        SystemCfg_finalizeResources(systemCfgH, NULL, NULL);
    }

    /* stop remote processor */
    if (startStat >= 0) {
        SystemCfg_stop(systemCfgH);
    }

    /* delete SystemCfg instance */
    if (systemCfgH != NULL) {
        SystemCfg_delete(&systemCfgH);
    }

    /* finalize used modules */
    SystemCfg_exit();

    /* report if error */
    if (status < 0) {
        System_printf("FAIL: example encountered errors\n");
    }
    return(status);
}
#undef FXNN
/*
 *  @(#) ti.sdo.rcm.examples.hello; 1,0,0,38; 11-2-2010 18:30:36; /db/atree/library/trees/fc/fc-n22x/src/ xlibrary

 */

