/*
 *  Copyright 2010 by Texas Instruments Incorporated.
 *
 */

/* 
 * Copyright (c) 2010, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

import xdc.runtime.IHeap;

/*!
 *  ======== ECPY ========
 *  EDMA functional layer library.
 */

@Template("./ECPY.xdt")
metaonly module ECPY {

    /*!
     *  ======= edma3BaseAddress =======
     *  EDMA3 Base Address for the device
     *  Not required to be set if this library
     *  is used alongwith RMAN and IRES_EDMA3CHAN.
     */
    config UInt edma3BaseAddress;

    /*!
     *  ======= paRamBaseAddress =======
     *  PaRam Base Address for the device
     *  Not required to be set if this library
     *  is used alongwith RMAN and IRES_EDMA3CHAN or for built for TI targets.
     */
    config UInt paRamBaseAddress;

    /*!
     *  ======= cpu ========
     *  Builds a non-EDMA, CPU only version of the library
     */
     config Bool cpu = false;

    /*!
     *  ======== instrumented ========
     *  Enable instrumentation in the ECPYCFG library.
     */
    config bool instrumented = false;
    
    config Bool enableLog;
    /*!
     *  ======== persistentAllocFxn ========
     *  Function for allocating persistent memory for ECPY's 
     *  implementation's internal objects and for allocating memory for 
     *  ECPY handle objects.
     *
     *  The signature of the persistent alloc function is:
     *      extern Bool persistentAllocFxn(IALG_MemRec * memTab, Int numRecs); 
     */
    config String persistentAllocFxn = null;
 
    /*!
     *  ======== persistentFreeFxn ========
     *  Function for freeing persistent memory allocated by ECPY.
     *
     *  The signature of hte persistentFreeFxn is:
     *
     *      extern Void persistentFreeFxn(IALG_MemRec *memTab, Int numRecs);
     */
    config String persistentFreeFxn = null;
}
/*
 *  @(#) ti.sdo.fc.ecpy; 1, 0, 0,10; 8-26-2010 17:48:49; /db/atree/mk/fc-n13x/src/
 */

/*
 *  @(#) ti.sdo.fc.ecpy; 1, 0, 0,54; 11-2-2010 18:25:29; /db/atree/library/trees/fc/fc-n22x/src/ xlibrary

 */

